@@ -41,15 +41,16 @@ methods :meth:`~ae.gui_app.MainAppBase.init_app` and :meth:`~HelpAppBase.ensure_
             framework_method_to_push_widget_to_top_most(widget)
             ...
 
-For to activate the help mode the widget to display the help texts have to be assigned to the property/attribute
-:attr:`~HelpAppBase.help_layout`::
+For to activate the help mode the widget to display the help texts have to be assigned to the main app attribute
+:attr:`~HelpAppBase.help_layout` and to the framework app property :attr:`~ae.kivy_app.FrameworkApp.help_layout` via
+the :meth:`~ae.gui_app.MainAppBase.change_observable` method::
 
-    main_app.help_layout = HelpScreenContainerOrWindow()
+    main_app.change_observable('help_layout', HelpScreenContainerOrWindow())
 
 The :attr:`~HelpAppBase.help_layout` property is also used as a flag of the help mode activity. By assigning `None` to
-this attribute the help mode will be deactivated::
+these attributes the help mode will be deactivated::
 
-    main_app.help_layout = None
+    main_app.change_observable('help_layout', None)
 
 Use the attribute :attr:`~HelpAppBase.help_activator` to provide and store a widget that allows the user to toggle the
 help mode activation. The :meth:`~HelpAppBase.help_display` is using it as fallback widget if no help target (or
@@ -176,7 +177,7 @@ A more complex example app demonstrating the features of this context help syste
 app tours
 =========
 
-The following classes are providing a solid fundament to implement tours for your app:
+The following classes provided by this portion building a solid fundament to implement tours for your app:
 
 * :class:`TourBase`
 * :class:`TourDropdownFromButton`
@@ -205,7 +206,7 @@ from ae.gui_app import (
     FLOW_KEY_SEP, flow_action, flow_class_name, flow_path_id, id_of_flow, update_tap_kwargs, MainAppBase)
 
 
-__version__ = '0.1.31'
+__version__ = '0.1.34'
 
 
 register_package_translations()
@@ -450,13 +451,407 @@ def widget_page_id(wid: Any) -> str:
     return page_id
 
 
+class TourBase:
+    """ abstract tour base class, automatically registering sub-classes as app tours.
+
+    sub-class this generic, UI-framework-independent base class to bundle pages of a tour and make sure that the
+    attr:`~TourBase.page_ids` and :attr:`~TourBase.page_data` attributes are correctly set. An UI-framework-dependent
+    tour overlay/layout instance, created and assigned to main_app.tour_layout, will automatically create an instance
+    of your tour-specific sub-class on tour start.
+    """
+    def __init_subclass__(cls, **kwargs):
+        """ register tour class; called on declaration of tour subclass. """
+        super().__init_subclass__(**kwargs)
+        register_tour_class(cls)
+
+    def __init__(self, main_app: 'HelpAppBase'):
+        super().__init__()
+        main_app.vpo(f"TourBase.__init__(): tour overlay={main_app.tour_layout}")
+        self._auto_switch_page_request = None
+        self._delayed_setup_layout_call = None
+        self._initial_page_data = None
+        self._saved_app_states: Dict[str, Any] = dict()
+
+        self.auto_switch_pages: Union[bool, int] = False
+        """ switch automatically tour pages within the seconds specified by :attr:`.page_data`['next_page_delay'].
+
+        set to `True`, `1` or `-1` for to automatically switch tour pages; `True` and `1` will switch to the next page
+        until the last page is reached, while `-1` will switch back to the previous pages until the first page is
+        reached;
+        `-1` and `1` automatically toggles at the first/last page the to other value (endless ping-pong until back/next
+        button gets pressed by the user).
+        """
+
+        self.page_data: Dict[str, Any] = dict(help_vars=dict())
+        """ page data with additional help variables (in `help_vars` key) and optional page text/layout/timing settings.
+
+        The class attribute values are default values for all tour pages and get individually overwritten by the i18n
+        translations attributes on tour page change via :meth:`.load_page_data`.
+
+        supported/implemented dict keys:
+
+        * `app_flow_delay`: time in seconds to wait until app flow change is completed (def=1.2, >0.9 for auto-width).
+        * `app_reset_delay`: time in seconds between app reset/close-popups and flow reset/prepare-tour-page (def=0.3).
+        * `back_text`: caption of tour previous page button (def=get_text('back')).
+        * `help_vars`: additional help variables, e.g. `help_translation` providing context help translation dict/text.
+        * `next_text`: caption of tour next page button (def=get_text('next')).
+        * `next_page_delay`: time in seconds to read the current page before next request_auto_page_switch() (def=9.6).
+        * `tip_text` or (empty string): tour page tooltip text fstring message text template.
+        """
+
+        self.page_ids: List[str] = list()
+        """ list of tour page ids, either initialized via this class attribute or dynamically. """
+
+        self.pages_explained_matcher: Dict[str, Union[Callable[[Any], bool], str]] = dict()
+        """ :meth:`MainAppBase.find_widget` match callable to determine explained widget of each tour page. """
+
+        self.page_idx: int = 0                      #: index to the current tour page (in :attr:`.page_ids`)
+        self.last_page_idx: Optional[int] = None    #: last tour page index (None on tour start)
+
+        self.main_app = main_app                    #: shortcut to main app instance
+        self.layout = main_app.tour_layout          #: tour overlay layout instance
+
+        self.top_popup = None                       #: top most popup widget (in app simulation)
+
+        self.backup_app_states()
+
+    def backup_app_states(self):
+        """ backup current states of this app, including flow. """
+        main_app = self.main_app
+        main_app.vpo("TourBase.backup_app_states")
+        self._saved_app_states = deepcopy(main_app.retrieve_app_states())
+
+    def cancel_auto_page_switch_request(self, reset: bool = True):
+        """ cancel auto switch callback if requested, called e.g. from tour layout/overlay next/back buttons. """
+        if self._auto_switch_page_request:
+            self._auto_switch_page_request.cancel()
+            self._auto_switch_page_request = None
+        if reset:
+            self.auto_switch_pages = False
+
+    def cancel_delayed_setup_layout_call(self):
+        """ cancel delayed setup layout call request. """
+        if self._delayed_setup_layout_call:
+            self._delayed_setup_layout_call.cancel()
+            self._delayed_setup_layout_call = None
+
+    @property
+    def last_page_id(self) -> Optional[str]:
+        """ determine last displayed tour page id. """
+        return None if self.last_page_idx is None else self.page_ids[self.last_page_idx]    # type: ignore # silly mypy
+
+    def load_page_data(self):
+        """ load page before switching to it (and maybe reload after preparing app flow and before setup of layout). """
+        page_idx = self.page_idx
+        page_cnt = len(self.page_ids)
+        assert 0 <= page_idx < page_cnt, f"page_idx ({page_idx}) has to be greater zero and less than {page_cnt}"
+        page_id = self.page_ids[page_idx]
+        if self._initial_page_data is None:     # reset page data to tour class default: dict(help_vars=dict(), ...)
+            self._initial_page_data = self.page_data
+            page_data = deepcopy(self.page_data)
+        else:
+            page_data = deepcopy(self._initial_page_data)
+
+        help_translation = tour_help_translation(page_id)
+        tour_translation = translation_short_help_id(id_of_tour_help(page_id))[0]
+        if help_translation:
+            if tour_translation:
+                page_data['help_vars']['help_translation'] = help_translation
+            else:
+                tour_translation = help_translation
+
+        page_data.update(tour_translation if isinstance(tour_translation, dict) else {'tip_text': tour_translation})
+
+        self.main_app.vpo(f"TourBase.load_page_data(): tour page{page_idx}/{page_cnt} id={page_id} data={page_data}")
+        self.page_data = page_data
+
+    def next_page(self):
+        """ switch to next tour page. """
+        self.teardown_app_flow()
+
+        ids = self.page_ids
+        assert self.page_idx + 1 < len(ids), f"TourBase.next_page missing {self.__class__.__name__}:{self.page_idx + 1}"
+        self.last_page_idx = self.page_idx
+        self.page_idx += 1
+        self.main_app.vpo(f"TourBase.next_page #{self.page_idx} id={ids[self.last_page_idx]}->{ids[self.page_idx]}")
+
+        self.setup_app_flow()
+
+    def prev_page(self):
+        """ switch to previous tour page. """
+        self.teardown_app_flow()
+
+        ids = self.page_ids
+        assert self.page_idx > 0, f"TourBase.prev_page wrong/missing page {self.__class__.__name__}:{self.page_idx - 1}"
+        self.last_page_idx = self.page_idx
+        self.page_idx -= 1
+        self.main_app.vpo(f"TourBase.prev_page #{self.page_idx} id={ids[self.last_page_idx]}->{ids[self.page_idx]}")
+
+        self.setup_app_flow()
+
+    def request_auto_page_switch(self):
+        """ initiate automatic switch to next tour page. """
+        self.cancel_auto_page_switch_request(reset=False)
+
+        next_idx = self.page_idx + self.auto_switch_pages
+        if not 0 <= next_idx < len(self.page_ids):
+            if self.auto_switch_pages is True:
+                self.cancel_auto_page_switch_request()  # only switch to next until last page reached
+                return
+            self.auto_switch_pages = -self.auto_switch_pages
+            next_idx += 2 * self.auto_switch_pages
+
+        main_app = self.main_app
+        delay = self.page_data.get('next_page_delay', 30.9)
+        main_app.vpo(f"TourOverlay.request_auto_page_switch from #{self.page_idx} to #{next_idx} delay={delay}")
+        self._auto_switch_page_request = main_app.call_method_delayed(
+            delay, self.prev_page if self.auto_switch_pages < 0 else self.next_page)
+
+    def reset_app_flow(self):
+        """ delayed reset of flow path/id for to be processed after dispatch of popups dismiss. """
+        self.main_app.vpo("TourBase.reset_app_flow")
+        # optionally set/overwrite current app flow, e.g. like:
+        # main_app = self.main_app
+        # main_app.change_app_state('flow_id', '')
+        # main_app.change_app_state('flow_path', list())
+        # main_app.refresh_all()
+
+    def reset_app_states(self):
+        """ reset app to a defined start to prepare a new tour start. """
+        main_app = self.main_app
+        main_app.vpo("TourBase.reset_app_states")
+
+        # close all popups and delay reset of flow path/id - to be done after dispatch of popups dismiss
+        main_app.close_popups()
+        main_app.call_method_delayed(self.page_data.get('app_reset_delay', 0.3), self.reset_app_flow)
+
+    def restore_app_states(self):
+        """ restore app states of this app - saved via :meth:`.backup_app_states`. """
+        main_app = self.main_app
+        main_app.vpo("TourBase.restore_app_states")
+        main_app.setup_app_states(self._saved_app_states)
+
+    def setup_app_flow(self):
+        """ setup app flow and load page data for to prepare a tour page. """
+        self.main_app.vpo(f"TourBase.setup_app_flow page_data={self.page_data}")
+
+        self.load_page_data()
+
+        app_flow_delay = self.page_data.get('app_flow_delay', 1.2)  # > 0.9 to complete auto width animation
+        self._delayed_setup_layout_call = self.main_app.call_method_delayed(app_flow_delay, self.setup_layout)
+
+    def setup_explained_widget(self):
+        """ determine and set the explained widget for the actual tour page. """
+        main_app = self.main_app
+        page_id = self.page_ids[self.page_idx]
+        if page_id in self.pages_explained_matcher:
+            matcher = self.pages_explained_matcher[page_id]
+            if isinstance(matcher, str):
+                matcher = try_eval(matcher, glo_vars=main_app.global_variables(layout=self.layout, tour=self))
+            wid = main_app.find_widget(matcher)
+            self.page_data['help_vars']['help_translation'] = tour_help_translation(widget_page_id(wid))
+        else:
+            wid = main_app.widget_by_flow_id(page_id)
+            if not wid:
+                wid = main_app.widget_by_app_state_name(page_id)
+        if wid:
+            self.layout.explained_widget = wid
+
+    def setup_layout(self):
+        """ setup/prepare tour overlay/layout after switch of tour page. """
+        self._delayed_setup_layout_call = None
+        main_app = self.main_app
+        layout = self.layout
+
+        try:
+            self.top_popup = main_app.popups_opened()[-1]
+        except IndexError:
+            self.top_popup = None
+
+        self.update_page_ids()
+        main_app.vpo(f"TourBase.setup_layout(): page id={self.page_ids[self.page_idx]}")
+
+        self.setup_explained_widget()
+        self.setup_texts()
+        layout.page_updated()
+
+        main_app.ensure_top_most_z_index(layout)
+
+        if self.auto_switch_pages:
+            self.request_auto_page_switch()
+
+    def setup_texts(self):
+        """ setup texts in tour layout from page_data. """
+        main_app = self.main_app
+        layout = self.layout
+        page_data = self.page_data
+        page_idx = self.page_idx
+
+        glo_vars = main_app.global_variables(layout=layout, tour=self)
+        if self.layout.explained_widget:
+            glo_vars['self'] = self.layout.explained_widget
+        if self.top_popup:
+            glo_vars['root'] = self.top_popup
+
+        help_vars = page_data['help_vars']
+        help_vars['self'] = layout.explained_widget
+
+        main_app.vpo(f"TourBase.setup_texts page_data={page_data}")
+
+        _txt = lambda _t: _t is not None and get_f_string(_t, glo_vars=glo_vars, loc_vars=help_vars) or ""  # noqa: E731
+
+        layout.title_text = _txt(page_data.get('title_text'))
+        layout.page_text = _txt(page_data.get('page_text'))
+
+        tip_text = page_data.get('tip_text', page_data.get(''))
+        if tip_text is None:
+            help_tra = help_vars.get('help_translation')
+            tip_text = help_tra.get('', "") if isinstance(help_tra, dict) else help_tra
+        layout.tip_text = _txt(tip_text)
+
+        layout.next_text = page_data.get('next_text', get_text('next')) if page_idx + 1 < len(self.page_ids) else ""
+        layout.prev_text = page_data.get('back_text', get_text('back')) if page_idx > 0 else ""
+
+    def start(self):
+        """ prepare app for tour start. """
+        self.main_app.vpo("TourBase.start")
+        self.reset_app_states()
+        self.setup_app_flow()
+
+    def stop(self):
+        """ stop/cancel tour. """
+        self.main_app.vpo("TourBase.stop")
+        self.teardown_app_flow()
+
+    def teardown_app_flow(self):
+        """ restore app flow and app states before tour finishing or before preparing/switching to prev/next page. """
+        self.main_app.vpo("TourBase.teardown_app_flow")
+        self.cancel_delayed_setup_layout_call()
+        self.cancel_auto_page_switch_request(reset=False)
+        self.restore_app_states()
+
+    def update_page_ids(self):
+        """ update/change page ids after app flow setup and before the tour overlay/layout setup.
+
+        override this method to change the page_ids in a running tour. don't forget to reload the page data (by calling
+        :meth:`.load_page_data`) if you remove/replace the current page, or to correct/decrement the attribute values
+        of :attr:`.last_page_idx` and :attr:`.page_idx` if you remove a page which is preceding the current page.
+        """
+        self.main_app.vpo(f"TourBase.update_page_ids {self.page_ids}")
+
+
+class TourDropdownFromButton(TourBase):
+    """ generic tour base class for to auto-explain a dropdown menu, starting with the button opening the dropdown. """
+    determine_page_ids = '_v_'
+
+    def setup_app_flow(self):
+        """ prevent opening of sub-menus and close of dropdown if dropdown child would open another dropdown. """
+        super().setup_app_flow()
+        if self.last_page_idx == 0 and self.page_idx == 1:
+            last_page_id = self.page_ids[self.last_page_idx]
+            if flow_action(last_page_id) == 'open':
+                self.main_app.change_flow(last_page_id, **update_tap_kwargs(self.layout.explained_widget))
+
+        elif self.last_page_idx == 1 and self.page_idx == 0:
+            if self.top_popup and flow_action(self.page_ids[self.page_idx]) == 'open':
+                self.top_popup.close()
+
+    def update_page_ids(self):
+        """ prepare all tour pages - first page is explaining opening dropdown button. """
+        super().update_page_ids()
+        page_ids = self.page_ids
+        if self.page_idx == 1 and page_ids[-1] == TourDropdownFromButton.determine_page_ids:
+            main_app = self.main_app
+            if not self.top_popup:
+                main_app.po("TourDropDownFromButton.update_page_ids: dropdown not opened")
+                return
+
+            children = main_app.widget_tourable_children_page_ids(self.top_popup)
+            if not children:
+                main_app.po(f"TourDropDownFromButton.update_page_ids missing tour-able child in {self.top_popup}")
+                return
+
+            page_ids.remove(TourDropdownFromButton.determine_page_ids)
+            page_ids.extend(children)
+
+            self.load_page_data()   # reload page data of this 2nd tour page (explaining the first dropdown child)
+
+
+# ====== app tours =============================================================
+
+_OPEN_USER_PREFERENCES_FLOW_ID = id_of_flow('open', 'user_preferences')
+
+
+class OnboardingTour(TourBase):
+    """ onboarding tour for first app start. """
+    def __init__(self, main_app: 'HelpAppBase'):
+        """ overridden to handle onboarding tour starts since app installation. """
+        started = main_app.get_var('onboarding_tour_started', default_value=0) + 1
+        main_app.set_var('onboarding_tour_started', started)    # persistently store in config variable
+
+        super().__init__(main_app)
+
+        self.page_ids.extend([
+            '', 'page_switching', 'responsible_layout', 'tip_help_intro', 'tip_help_tooltip', 'layout_font_size',
+            'tour_end', 'user_registration'])
+
+        self.pages_explained_matcher.update(dict(
+            tip_help_intro=lambda widget: widget.__class__.__name__ == 'HelpToggler',
+            tip_help_tooltip=lambda widget: getattr(widget, 'tap_flow_id', None) == _OPEN_USER_PREFERENCES_FLOW_ID,
+            layout_font_size=lambda widget: getattr(widget, 'app_state_name', None) == 'font_size',
+        ))
+
+        if started > main_app.get_var('onboarding_tour_max_started', default_value=9):
+            # this would remove welcome and base pages, unreachable for the user:  ids[:] = ids[ids.index('tour_end'):]
+            self.page_idx = self.page_ids.index('tour_end')   # .. instead jump to last page before user registration
+
+    def setup_app_flow(self):
+        """ overridden to open user preferences dropdown in responsible_layout tour page. """
+        super().setup_app_flow()
+        page_id = self.page_ids[self.page_idx]
+        if page_id == 'layout_font_size':
+            main_app = self.main_app
+            flow_id = _OPEN_USER_PREFERENCES_FLOW_ID
+            wid = main_app.widget_by_flow_id(flow_id)
+            main_app.change_flow(flow_id, **update_tap_kwargs(wid))
+
+        elif page_id == 'user_registration':
+            self.layout.stop_tour()
+            self.main_app.change_flow(id_of_flow('open', 'user_name_editor'))
+
+    def teardown_app_flow(self):
+        """ overridden to close the opened user preferences dropdown on leaving responsible_layout tour page. """
+        super().teardown_app_flow()
+        if self.top_popup and self.page_ids[self.page_idx] == 'layout_font_size':
+            self.top_popup.close()
+
+    def update_page_ids(self):
+        """ overridden to remove page (only showing once). """
+        super().update_page_ids()
+        if self.last_page_id == 'page_switching':
+            self.page_ids.remove('page_switching')
+            if self.last_page_idx == self.page_idx - 1:
+                self.last_page_idx -= 1
+                self.page_idx -= 1
+
+
+class UserPreferencesTour(TourDropdownFromButton):
+    """ user preferences menu tour. """
+
+    def __init__(self, main_app: 'HelpAppBase'):
+        super().__init__(main_app)
+
+        self.auto_switch_pages = 1
+        self.page_data['next_page_delay'] = 3.6
+        self.page_ids.extend([_OPEN_USER_PREFERENCES_FLOW_ID, TourDropdownFromButton.determine_page_ids])
+
+
 class HelpAppBase(MainAppBase, ABC):
     """ main app help base class. """
-
-    # additional instance attributes
     displayed_help_id: str = ''                 #: message id of currently explained/focused target widget in help mode
     help_activator: Any = None                  #: help mode de-/activator button widget
     help_layout: Optional[Any] = None           #: help text container widget in active help mode else None
+    onboarding_tour_class: Type[OnboardingTour] = OnboardingTour    #: default onboarding app tour class
     tour_layout: Optional[Any] = None           #: tour layout/overlay widget in active tour mode else None
     tour_overlay_class: Optional[Type] = None   #: UI-framework-specific tour overlay class, set by main app sub-class
 
@@ -704,9 +1099,9 @@ class HelpAppBase(MainAppBase, ABC):
         return True
 
     def save_app_states(self) -> str:
-        """ overwrite MainAppBase method to auto-cancel active tour on app pause/close. """
+        """ override MainAppBase method to not overwrite app states if app tour is active. """
         if self.tour_layout:
-            self.tour_layout.stop_tour()
+            return "running app tour prevent to save app states into config file"   # was: self.tour_layout.stop_tour()
         return super().save_app_states()
 
     def start_app_tour(self, tour_class: Optional[Type['TourBase']] = None) -> bool:
@@ -742,353 +1137,3 @@ class HelpAppBase(MainAppBase, ABC):
                     if page_id and page_id not in tourable_children:
                         tourable_children.append(page_id)
         return tourable_children
-
-
-class TourBase:
-    """ abstract tour base class, automatically registering sub-classes as app tours.
-
-    Sub-class this generic, UI-framework-independent base class to bundle pages of a tour and make sure that the
-    attr:`~TourBase.page_ids` and :attr:`~TourBase.page_idx` attributes are correctly set. An UI-framework-dependent
-    tour overlay/layout instance, created and assigned to main_app.tour_layout, will automatically create an instance
-    of your tour-specific sub-class on tour start.
-    """
-    auto_switch_pages: bool = False
-    """ if True switch automatically tour pages within the seconds specified by :attr:`.page_data`['next_page_delay'].
-    """
-
-    page_data: Dict[str, Any] = dict(help_vars=dict())
-    """ page data with additional help variables (in `help_vars` key) and optional page text/layout/timing settings.
-
-    The class attribute values are default values for all tour pages and get individually overwritten by the i18n
-    translations attributes on tour page change via :meth:`.load_page_data`.
-
-    supported/implemented dict keys:
-
-    * `app_flow_delay`: time in seconds to wait until app flow change is fully completed (def=1.2, >0.9 for auto-width).
-    * `app_reset_delay`: time in seconds between app reset/close-popups and flow reset/prepare-tour-page (def=0.3).
-    * `back_text`: caption of tour previous page button (def=get_text('back')).
-    * `help_vars`: additional help variables, e.g. `help_translation` providing context help translation dict/text.
-    * `next_text`: caption of tour next page button (def=get_text('next')).
-    * `next_page_delay`: time in seconds to read the current tour page before next auto_switch_next_page() (def=9.6).
-    * `tip_text` or (empty string): tour page tooltip text fstring message text template.
-    """
-
-    page_ids: List[str] = list()
-    """ tour page ids, either initialized via this class attribute or dynamically. """
-
-    pages_explained_matcher: Dict[str, Union[Callable[[Any], bool], str]] = dict()
-    """ :meth:`MainAppBase.find_widget` match callable to determine explained widget of each tour page. """
-
-    def __init_subclass__(cls, **kwargs):
-        """ register tour class; called on declaration of tour subclass. """
-        super().__init_subclass__(**kwargs)
-        register_tour_class(cls)
-
-    def __init__(self, main_app: HelpAppBase):
-        main_app.vpo(f"TourBase.__init__(): tour overlay={main_app.tour_layout}")
-        self._auto_switch_next_page_request = None
-        self._delayed_setup_layout_call = None
-        self._saved_app_states: Dict[str, Any] = dict()
-
-        self.main_app = main_app
-        assert main_app.tour_layout
-        self.layout = main_app.tour_layout
-
-        self.page_ids = self.page_ids.copy()        # copy page ids from class to instance attribute
-        self.page_idx = 0
-        self.last_page_idx = None
-
-        self.backup_app_states()
-
-    def auto_switch_cancel(self):
-        """ cancel auto switch callback if requested. """
-        if self._auto_switch_next_page_request:
-            self._auto_switch_next_page_request.cancel()
-            self._auto_switch_next_page_request = None
-        self.auto_switch_pages = False
-
-    def auto_switch_next_page(self):
-        """ initiate automatic switch to next tour page. """
-        main_app = self.main_app
-        delay = self.page_data.get('next_page_delay', 9.6)
-        main_app.vpo(f"TourOverlay.auto_switch_next_page from page #{self.page_idx} delay={delay}")
-        if self.page_idx + 1 < len(self.page_ids):
-            self._auto_switch_next_page_request = main_app.call_method_delayed(delay, self.next_page)
-
-    def backup_app_states(self):
-        """ backup current states of this app, including flow. """
-        main_app = self.main_app
-        main_app.vpo("TourBase.backup_app_states")
-        self._saved_app_states = deepcopy(main_app.retrieve_app_states())
-
-    @property
-    def last_page_id(self) -> Optional[str]:
-        """ determine last displayed tour page id. """
-        return None if self.last_page_idx is None else self.page_ids[self.last_page_idx]    # type: ignore # silly mypy
-
-    def load_page_data(self):
-        """ load page before switching to it (and maybe reload after preparing app flow and before setup of layout). """
-        page_idx = self.page_idx
-        page_cnt = len(self.page_ids)
-        assert 0 <= page_idx < page_cnt, f"page_idx ({page_idx}) has to be greater zero and less than {page_cnt}"
-        page_id = self.page_ids[page_idx]
-        page_data = deepcopy(self.__class__.page_data)        # reset to tour class default: dict(help_vars=dict(), ...)
-
-        help_translation = tour_help_translation(page_id)
-        tour_translation = translation_short_help_id(id_of_tour_help(page_id))[0]
-        if help_translation:
-            if tour_translation:
-                page_data['help_vars']['help_translation'] = help_translation
-            else:
-                tour_translation = help_translation
-
-        page_data.update(tour_translation if isinstance(tour_translation, dict) else {'tip_text': tour_translation})
-
-        self.main_app.vpo(f"TourBase.load_page_data(): tour page{page_idx}/{page_cnt} id={page_id} data={page_data}")
-        self.page_data = page_data
-
-    def next_page(self):
-        """ switch to next tour page. """
-        self.teardown_page()
-
-        ids = self.page_ids
-        assert self.page_idx + 1 < len(ids), f"TourBase.next_page missing {self.__class__.__name__}:{self.page_idx + 1}"
-        self.last_page_idx = self.page_idx
-        self.page_idx += 1
-        self.main_app.vpo(f"TourBase.next_page #{self.page_idx} id={ids[self.last_page_idx]}->{ids[self.page_idx]}")
-
-        self.setup_page()
-
-    def prev_page(self):
-        """ switch to previous tour page. """
-        self.auto_switch_cancel()
-        self.teardown_page()
-
-        ids = self.page_ids
-        assert self.page_idx > 0, f"TourBase.prev_page wrong/missing page {self.__class__.__name__}:{self.page_idx - 1}"
-        self.last_page_idx = self.page_idx
-        self.page_idx -= 1
-        self.main_app.vpo(f"TourBase.prev_page #{self.page_idx} id={ids[self.last_page_idx]}->{ids[self.page_idx]}")
-
-        self.setup_page()
-
-    def reset_app_flow(self):
-        """ delayed reset of flow path/id for to be processed after dispatch of popups dismiss. """
-        self.main_app.vpo("TourBase.reset_app_flow")
-        # optionally set/overwrite current app flow, e.g. like:
-        # main_app = self.main_app
-        # main_app.change_app_state('flow_id', '')
-        # main_app.change_app_state('flow_path', list())
-        # main_app.refresh_all()
-
-    def reset_app_states(self):
-        """ reset app to a defined start to prepare a new tour start. """
-        main_app = self.main_app
-        main_app.vpo("TourBase.reset_app_states")
-
-        # close all popups and delay reset of flow path/id - to be done after dispatch of popups dismiss
-        main_app.close_popups()
-        main_app.call_method_delayed(self.page_data.get('app_reset_delay', 0.3), self.reset_app_flow)
-
-    def restore_app_states(self):
-        """ restore app states of this app - saved via :meth:`.backup_app_states`. """
-        main_app = self.main_app
-        main_app.vpo("TourBase.restore_app_states")
-        main_app.setup_app_states(self._saved_app_states)
-
-    def setup_explained_widget(self):
-        """ determine and set the explained widget for the actual tour page. """
-        main_app = self.main_app
-        page_id = self.page_ids[self.page_idx]
-        if page_id in self.pages_explained_matcher:
-            matcher = self.pages_explained_matcher[page_id]
-            if isinstance(matcher, str):
-                matcher = try_eval(matcher, glo_vars=main_app.global_variables(layout=self.layout, tour=self))
-            wid = main_app.find_widget(matcher)
-            self.page_data['help_vars']['help_translation'] = tour_help_translation(widget_page_id(wid))
-        else:
-            wid = main_app.widget_by_flow_id(page_id)
-            if not wid:
-                wid = main_app.widget_by_app_state_name(page_id)
-        if wid:
-            self.layout.explained_widget = wid
-
-    def setup_layout(self):
-        """ setup/prepare tour overlay/layout after switch of tour page. """
-        self._delayed_setup_layout_call = None
-        main_app = self.main_app
-        layout = self.layout
-
-        main_app.ensure_top_most_z_index(layout)
-
-        self.update_page_ids()
-        main_app.vpo(f"TourBase.setup_layout(): page id={self.page_ids[self.page_idx]}")
-
-        self.setup_explained_widget()
-        self.setup_texts()
-        layout.page_updated()
-
-        if self.auto_switch_pages:
-            self.auto_switch_next_page()
-
-    def setup_page(self):
-        """ setup tour overlay to load data for to display new tour page. """
-        self.main_app.vpo(f"TourBase.setup_page page_data={self.page_data}")
-
-        self.load_page_data()
-
-        app_flow_delay = self.page_data.get('app_flow_delay', 1.2)  # > 0.9 to complete auto width animation
-        self._delayed_setup_layout_call = self.main_app.call_method_delayed(app_flow_delay, self.setup_layout)
-
-    def setup_texts(self):
-        """ setup texts in tour layout from page_data. """
-        main_app = self.main_app
-        layout = self.layout
-        page_data = self.page_data
-        page_idx = self.page_idx
-
-        glo_vars = main_app.global_variables(layout=layout, tour=self)
-        help_vars = page_data['help_vars']
-        help_vars['self'] = layout.explained_widget
-
-        main_app.vpo(f"TourBase.setup_texts page_data={page_data}")
-
-        _txt = lambda _t: _t is not None and get_f_string(_t, glo_vars=glo_vars, loc_vars=help_vars) or ""  # noqa: E731
-
-        layout.title_text = _txt(page_data.get('title_text'))
-        layout.page_text = _txt(page_data.get('page_text'))
-
-        tip_text = page_data.get('tip_text', page_data.get(''))
-        if tip_text is None:
-            help_tra = help_vars.get('help_translation')
-            tip_text = help_tra.get('', "") if isinstance(help_tra, dict) else help_tra
-        layout.tip_text = _txt(tip_text)
-
-        layout.next_text = page_data.get('next_text', get_text('next')) if page_idx + 1 < len(self.page_ids) else ""
-        layout.prev_text = page_data.get('back_text', get_text('back')) if page_idx > 0 else ""
-
-    def start(self):
-        """ prepare app for tour start. """
-        self.main_app.vpo("TourBase.start")
-        self.reset_app_states()
-        self.page_idx = 0
-        self.last_page_idx = None
-        self.setup_page()
-
-    def stop(self):
-        """ stop/cancel tour. """
-        self.main_app.vpo("TourBase.stop")
-        if self._delayed_setup_layout_call:
-            self._delayed_setup_layout_call.cancel()
-            self._delayed_setup_layout_call = None
-        self.auto_switch_cancel()
-        self.teardown_page()
-
-    def teardown_page(self):
-        """ restore tour page state before switch to prev/next page or tour finishing. """
-        self.main_app.vpo("TourBase.teardown_page")
-        self.restore_app_states()
-
-    def update_page_ids(self):
-        """ update/change page ids after app flow setup and before the tour overlay/layout setup.
-
-        override this method to change the page_ids in a running tour. don't forget to reload the page data (by calling
-        :meth:`.load_page_data`) if you remove/replace the current page, or to correct/decrement the attribute values
-        of :attr:`.last_page_idx` and :attr:`.page_idx` if you remove a page which is preceding the current page.
-        """
-        self.main_app.vpo(f"TourBase.update_page_ids {self.page_ids}")
-
-
-class TourDropdownFromButton(TourBase):
-    """ generic tour base class for to auto-explain a dropdown menu, starting with the button opening the dropdown. """
-    determine_page_ids = '_v_'
-
-    def setup_page(self):
-        """ prevent opening of sub-menus and close of dropdown if dropdown child would open another dropdown. """
-        super().setup_page()
-        if self.last_page_idx == 0 and self.page_idx == 1:
-            last_page_id = self.page_ids[self.last_page_idx]
-            if flow_action(last_page_id) == 'open':
-                self.main_app.change_flow(last_page_id, **update_tap_kwargs(self.layout.explained_widget))
-
-        elif self.last_page_idx == 1 and self.page_idx == 0:
-            if flow_action(self.page_ids[self.page_idx]) == 'open':
-                self.main_app.popups_opened()[-1].close()
-
-    def update_page_ids(self):
-        """ prepare all tour pages - first page is explaining opening dropdown button. """
-        super().update_page_ids()
-        page_ids = self.page_ids
-        if self.page_idx == 1 and page_ids[-1] == TourDropdownFromButton.determine_page_ids:
-            main_app = self.main_app
-            popups = main_app.popups_opened()
-            if not popups:
-                main_app.po("TourDropDownFromButton.update_page_ids: dropdown not opened")
-                return
-
-            children = main_app.widget_tourable_children_page_ids(popups[-1])
-            if not children:
-                main_app.po(f"TourDropDownFromButton.update_page_ids missing tour-able child in {popups[-1]}")
-                return
-
-            page_ids.remove(TourDropdownFromButton.determine_page_ids)
-            page_ids.extend(children)
-
-            self.load_page_data()   # reload page data of this 2nd tour page (explaining the first dropdown child)
-
-
-# ====== app tours =============================================================
-
-_OPEN_USER_PREFERENCES_FLOW_ID = id_of_flow('open', 'user_preferences')
-
-
-class OnboardingTour(TourBase):
-    """ onboarding tour for first app start. """
-    page_ids = ['', 'page_switching', 'tip_help_intro', 'tip_help_tooltip', 'responsible_layout', 'layout_font_size',
-                'tour_end', 'user_registration']
-
-    pages_explained_matcher = dict(
-        tip_help_intro=lambda widget: widget.__class__.__name__ == 'HelpToggler',
-        tip_help_tooltip=lambda widget: getattr(widget, 'tap_flow_id', None) == _OPEN_USER_PREFERENCES_FLOW_ID,
-        layout_font_size=lambda widget: getattr(widget, 'app_state_name', None) == 'font_size',
-    )
-
-    def setup_page(self):
-        """ overridden to open user preferences dropdown in responsible_layout tour page. """
-        super().setup_page()
-
-        page_id = self.page_ids[self.page_idx]
-        if page_id == 'layout_font_size':
-            main_app = self.main_app
-            flow_id = _OPEN_USER_PREFERENCES_FLOW_ID
-            wid = main_app.widget_by_flow_id(flow_id)
-            main_app.change_flow(flow_id, **update_tap_kwargs(wid))
-
-        elif page_id == 'user_registration':
-            self.layout.stop_tour()
-            self.main_app.change_flow(id_of_flow('open', 'user_name_editor'))
-
-    def teardown_page(self):
-        """ overridden to close the opened user preferences dropdown on leaving responsible_layout tour page. """
-        super().teardown_page()
-
-        if self.page_ids[self.page_idx] == 'layout_font_size':
-            popups = self.main_app.popups_opened()
-            if popups:
-                popups[-1].close()
-
-    def update_page_ids(self):
-        """ overridden to remove page (only showing once). """
-        super().update_page_ids()
-        if self.last_page_id == 'page_switching':
-            self.page_ids.remove('page_switching')
-            if self.last_page_idx == self.page_idx - 1:
-                self.last_page_idx -= 1
-                self.page_idx -= 1
-
-
-class UserPreferencesTour(TourDropdownFromButton):
-    """ user preferences menu tour. """
-    auto_switch_pages = True
-    page_data = dict(help_vars=dict(), next_page_delay=3.6)
-    page_ids = [_OPEN_USER_PREFERENCES_FLOW_ID, TourDropdownFromButton.determine_page_ids]
